#include "stdafx.h"

const char* DefaultUser="root";
const char* DefaultPassword="202409";
const char* DefaultDatabase="nicktest";
const int DefaultPort=3306;

bool MySqlHelper::init()
{
	char ipAddress[32];

	if (!getIpAddress(ipAddress))
	{
		return false;
	}

	if (mysql_init(&mysql)==NULL)
	{
		return false;
	}
	if (mysql_real_connect(&mysql, ipAddress, DefaultUser, DefaultPassword,
		DefaultDatabase, DefaultPort, NULL, 0)==NULL)
	{
		mysql_close(&mysql);
		return false;
	}
	
	return true;
}


void MySqlHelper::assembleEscapeStr(char* dest, const char* src, int srcSize, int& firstLength)
{
	int dataLength;
	dest[firstLength++]='"';	

	dataLength = mysql_real_escape_string(&mysql, dest + firstLength, src, srcSize);

	firstLength += dataLength;
	dest[firstLength++]='"'; 		
}


bool MySqlHelper::updateUrlStatus(DomainNameType& domainName, LPSTR tableName, int oldStatus, int newStatus)
{
	int firstLength =0, srcLength = 0;

	sprintf(sqlBuffer, "update %s set %sStatus=%d where %sStatus=%d and %s=", tableName, 
		tableName, newStatus, tableName, oldStatus, tableName);

	firstLength = strlen(sqlBuffer);
	srcLength = _tcslen(domainName.nameBuffer) *sizeof(TCHAR);

	assembleEscapeStr(sqlBuffer, (LPCSTR)domainName.nameBuffer, srcLength, firstLength);
	if (mysql_real_query(&mysql, sqlBuffer, firstLength))
	{
		printf("query error %s\n", mysql_error(&mysql));
		return false;
	}
	return mysql_affected_rows(&mysql) == 1;	
}


bool MySqlHelper::doInsertNewLink(DomainNameType& hostName, char* tableName, int status)
{
	int hostLength= 0;
	int firstLength = 0;
	bool result = false;
	hostLength = _tcslen(hostName.nameBuffer)*sizeof(TCHAR);

	sprintf(sqlBuffer, "insert into %s(%sTime, %sLength, %sStatus, %s) values(now(), %u, %d,",
		tableName, tableName, tableName, tableName, tableName, hostLength, status);
	firstLength = strlen(sqlBuffer);
	assembleEscapeStr(sqlBuffer, (char*)hostName.nameBuffer, hostLength, firstLength);
	sqlBuffer[firstLength++]=')';
	if (!mysql_real_query(&mysql, sqlBuffer, firstLength))
	{
		if (mysql_affected_rows(&mysql) == 1)
		{
			result = true;
		}
	}

	return result;
}



bool MySqlHelper::insertNewLink(DomainNameType& domainName, char* tableName, char* fieldName, bool isImage)
{
	int firstLength =0;
	int srcLength = 0;
	DomainNameType hostName;
	int hostLength =0;
	bool result = false;
	int status = isImage? 1 : (-1);
	srcLength = _tcslen(domainName.nameBuffer)*sizeof(TCHAR);
	if (doInsertNewLink(domainName, tableName, 0))
	{	
		result = true;

	}
	if (HtmlParser::retrieveCurrentHost(domainName.nameBuffer, hostName, hostLength))
	{
		insertNewHost(hostName, status);		
	}
	return result;
}

MySqlHelper::~MySqlHelper()
{
	mysql_close(&mysql);
}
		
bool MySqlHelper::doFetchResult(MYSQL_RES*& res)
{
	if (mysql_real_query(&mysql, sqlBuffer, strlen(sqlBuffer)))
	{
		printf("query error %s\n", mysql_error(&mysql));	
		return false;
	}

	if ((res=mysql_store_result(&mysql))==NULL)
	{
		printf("store result error %s\n", mysql_error(&mysql));			
		return false;
	}
	return true;
}

bool MySqlHelper::fetchSingleResult(LPSTR buffer, int& bufferLength)
{
	MYSQL_ROW row;
	MYSQL_RES* res = NULL;
	unsigned long * lengths;
	
	int len = 0;
	bool result = false;	

	if (doFetchResult(res))
	{	
		if (mysql_num_rows(res) == 1)
		{
			if ((row = mysql_fetch_row(res))!= NULL)
			{	
				if ((lengths = mysql_fetch_lengths(res))!=NULL)
				{			
					len = lengths[0];	
					if (len < bufferLength)
					{			
						memcpy(buffer, row[0], len);
						bufferLength = len;
						buffer[len]='\0';
						result = true;
					}
					else
					{
						printf("error! link length too long %d\n", len);
					}
				}			
			}
		}
		mysql_free_result(res);			
	}
	return result;	 	
}

bool MySqlHelper::retrieveAvailableLink(DomainNameType& domainName, LPCSTR tableType)
{
	bool result = false;
	int bufferLength = MaxDomainNameLength;
	//sprintf(sqlBuffer, "select min(%s) from %s where %sStatus =0", tableType, 
	//	tableType, tableType);

	sprintf(sqlBuffer, "select %s from %s where %sStatus <=0 and %sLength>=%d limit 1", tableType, 
		tableType, tableType, tableType, rand()%1024+20);


	if(fetchSingleResult((LPSTR)domainName.nameBuffer, bufferLength))
	{
		if (bufferLength%sizeof(TCHAR) == 0)
		{
			bufferLength/=sizeof(TCHAR);
			domainName.nameBuffer[bufferLength] = _T('\0');
			result = true;
		}		
	}
	return result;
}

bool MySqlHelper::retrieveAvailableLinkNumber(int& number, LPCSTR tableType)
{
	char buffer[32];
	int bufferLength=32;
	sprintf(sqlBuffer, "select count(*) from %s where %sStatus <= 0", tableType, 
		tableType);
	if (fetchSingleResult(buffer, bufferLength))
	{
		buffer[bufferLength]='\0';
		if (sscanf(buffer, "%d", &number)==1)
		{
			return true;
		}
	}
	return false;
}

bool MySqlHelper::setLinkToReading(DomainNameType& domainName, LPCSTR tableType)
{
	bool result = false;	
	int srcLength, destLength;

	srcLength = _tcslen(domainName.nameBuffer)*sizeof(TCHAR);

		
	sprintf(sqlBuffer, "update %s set %sstatus = 1 where %sStatus<=0 and %s=", tableType, 
		tableType, tableType, tableType);
	destLength = strlen(sqlBuffer);
	assembleEscapeStr(sqlBuffer, (LPCSTR)domainName.nameBuffer, srcLength, destLength);
	sqlBuffer[destLength]= '\0';

	if (mysql_real_query(&mysql, sqlBuffer, destLength))
	{
		printf("query error %s\n", mysql_error(&mysql));	
		return false;
	}
	return mysql_affected_rows(&mysql) == 1;	
}


bool MySqlHelper::escapeString(LPSTR buffer, int& bufferLength, LPCSTR src, int srcSize)
{
	if (srcSize*2 >= bufferLength)
	{
		return false;
	}
	bufferLength = mysql_real_escape_string(&mysql, buffer, src, srcSize);

	return true;
}

bool MySqlHelper::setLinkToFinished(DomainNameType& domainName, LPCSTR tableType, int newStatus)
{
	int srcLength, destLength;
	int points = 0;
	bool result = false;
	// we must make sure it is an existing link. 
	sprintf(sqlBuffer, "update %s set %sStatus=%d where %sStatus=1 and %s=", tableType, 
		 tableType, newStatus, tableType, tableType);
	srcLength = _tcslen(domainName.nameBuffer) * sizeof(TCHAR);
	destLength = strlen(sqlBuffer);

	assembleEscapeStr(sqlBuffer, (LPCSTR)domainName.nameBuffer, srcLength, destLength);
	sqlBuffer[destLength] = '\0';

	if (mysql_real_query(&mysql, sqlBuffer, destLength))
	{
		printf("query error %s\n", mysql_error(&mysql));			
	}
	else
	{
		result = mysql_affected_rows(&mysql) == 1;
		if (result)
		{
			switch (newStatus)
			{
			case LinkStatusFinished:
				points = -1;
				break;
			case LinkStatusDead:
				points = -10;
				break;
			case LinkStatusImage:
				points = 2;
				break;
			}
			insertNewHost(domainName, points);
		}
	}
	return result;
}

bool MySqlHelper::insertNewHost(DomainNameType& hostName, int status)
{
	bool result = false;

	if (!doInsertNewLink(hostName, "myHost", status))
	{		
		result = updateHostNameStatus(hostName, status);					
	}
	return result;
}



bool MySqlHelper::updateHostNameStatus(DomainNameType& domainName, int points)
{
	int hostLength = 0;
	int firstLength = 0;

	//HtmlParser::retrieveCurrentHost(domainName.nameBuffer, hostName, hostLength);
	hostLength = _tcslen(domainName.nameBuffer)*sizeof(TCHAR);
	sprintf(sqlBuffer, "update myHost set myHostStatus=%d+myHostStatus where myHost=", points);
	firstLength = strlen(sqlBuffer);
	assembleEscapeStr(sqlBuffer, (char*)domainName.nameBuffer, hostLength, firstLength);
	if (mysql_real_query(&mysql, sqlBuffer, firstLength))
	{
		printf("query error %s\n", mysql_error(&mysql));
		return false;
	}
	return mysql_affected_rows(&mysql) == 1;
}


bool MySqlHelper::fetchStatus(LPCTSTR linkName, LPCSTR tableType, int&status)
{
	int srcLength, destLength;
	char buffer[32];
	int bufferLength =32;
	bool result = false;
	sprintf(sqlBuffer, "select %sStatus from %s where %s=", tableType, 
		tableType, tableType);
	srcLength = _tcslen(linkName) * sizeof(TCHAR);
	destLength = strlen(sqlBuffer);

	assembleEscapeStr(sqlBuffer, (LPCSTR)linkName, srcLength, destLength);
	sqlBuffer[destLength] = '\0';

	if (fetchSingleResult(buffer, bufferLength))
	{
		if (sscanf(buffer, "%d", &status)==1)
		{
			result = true;
		}
	}

	return result;
}










